﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace CodeBreaker
{
    public partial class MainWindow : Window
    {

        // The simulated code to be broken
        private string Code;
        // The list of Label controls that show the characters of the code being broken.
        private List<TextBlock> OutputCharLabels;

        // The list of ProgressBar controls that show the progress of the character being decoded
        private List<ProgressBar> prloProgressChar;


        private readonly BackgroundWorker bakCodebreaker = new BackgroundWorker();
        private readonly BackgroundWorker bakCodebreaker2 = new BackgroundWorker();
        private readonly BackgroundWorker bakCodebreaker3 = new BackgroundWorker();
        private readonly BackgroundWorker bakCodebreaker4 = new BackgroundWorker();

        public MainWindow()
        {
            InitializeComponent();

            //Register the DoWork event handler.
            bakCodebreaker.DoWork += bakCodebreaker_DoWork;
            bakCodebreaker2.DoWork += bakCodebreaker2_DoWork;
            bakCodebreaker3.DoWork += bakCodebreaker3_DoWork;
            bakCodebreaker4.DoWork += bakCodebreaker4_DoWork;

            //Register the ProgressChanged event handlers.
            bakCodebreaker.ProgressChanged += bakCodebreaker_ProgressChanged;
            bakCodebreaker2.ProgressChanged += bakCodebreaker2_ProgressChanged;
            bakCodebreaker3.ProgressChanged += bakCodebreaker3_ProgressChanged;
            bakCodebreaker4.ProgressChanged += bakCodebreaker4_ProgressChanged;

            //Register the RunWorkerCompleted event handler.
            bakCodebreaker.RunWorkerCompleted += bakCodebreaker_RunWorkerCompleted;
            bakCodebreaker2.RunWorkerCompleted += bakCodebreaker2_RunWorkerCompleted;
            bakCodebreaker3.RunWorkerCompleted += bakCodebreaker3_RunWorkerCompleted;
            bakCodebreaker4.RunWorkerCompleted += bakCodebreaker4_RunWorkerCompleted;

            //Tell the background worker to report progress.
            bakCodebreaker.WorkerReportsProgress = true;
            bakCodebreaker2.WorkerReportsProgress = true;
            bakCodebreaker3.WorkerReportsProgress = true;
            bakCodebreaker4.WorkerReportsProgress = true;

            //Tell the background worker to support cancellation.
            bakCodebreaker.WorkerSupportsCancellation = true;
            bakCodebreaker2.WorkerSupportsCancellation = true;
            bakCodebreaker3.WorkerSupportsCancellation = true;
            bakCodebreaker4.WorkerSupportsCancellation = true;

            // Generate a random code to be broken
            SimulateCodeGeneration();
            // Create a new list of Label controls that show the characters of the code being broken.
            OutputCharLabels = new List<TextBlock>(4);
            // Add the Label controls to the List
            OutputCharLabels.Add(txtOutput1);
            OutputCharLabels.Add(txtOutput2);
            OutputCharLabels.Add(txtOutput3);
            OutputCharLabels.Add(txtOutput4);

            // Create a new list of ProgressBar controls that show the progress of each character of the code being broken
            prloProgressChar = new List<ProgressBar>(4);
            // Add the ProgressBar controls to the list
            prloProgressChar.Add(pgbProgressChar1);
            prloProgressChar.Add(pgbProgressChar2);
            prloProgressChar.Add(pgbProgressChar3);
            prloProgressChar.Add(pgbProgressChar4);

            //Hide the Stop button initially.
            btnStop.IsEnabled = false;

            // Hide the fishes game and show the Codebreaker
            showCodeBreaker();

        }

        private void SimulateCodeGeneration()
        {
            // A Random number generator
            Random loRandom = new Random();
            // The char position being generated
            int i;

            Code = "";

            for (i = 0; i <= 4; i++)
            {
                // Generate a Random Unicode char for each of the 4 positions.
                Code += (char)(loRandom.Next(65535));
            }
        }

        private void setFishesVisibility(System.Windows.Visibility pbValue)
        {
            // Change the visibility of the controls related to the fishes game.
            imgFish1.Visibility = pbValue;
            imgFish2.Visibility = pbValue;
            imgFish3.Visibility = pbValue;
            txtFishGame.Visibility = pbValue;
            btnGameOver.Visibility = pbValue;
        }

        private void setCodeBreakerVisibility(System.Windows.Visibility pbValue)
        {
            // Change the visibility of the controls related to the code breaking procedure.
            imgSkull.Visibility = pbValue;
            imgAgent.Visibility = pbValue;
            txtCodeBreaker.Visibility = pbValue;
            txtNumber1.Visibility = pbValue;
            txtNumber2.Visibility = pbValue;
            txtNumber3.Visibility = pbValue;
            txtNumber4.Visibility = pbValue;
            txtOutput1.Visibility = pbValue;
            txtOutput2.Visibility = pbValue;
            txtOutput3.Visibility = pbValue;
            txtOutput4.Visibility = pbValue;
            btnStart.Visibility = pbValue;
            btnHide.Visibility = pbValue;

            // Change the visibility of the controls related to the progress of the code breaking procedure.
            pgbProgressChar1.Visibility = pbValue;
            pgbProgressChar2.Visibility = pbValue;
            pgbProgressChar3.Visibility = pbValue;
            pgbProgressChar4.Visibility = pbValue;

            btnStop.Visibility = pbValue;

        }

        private void showFishes()
        {
            // Hide all the controls related to the code breaking procedure.
            setCodeBreakerVisibility(System.Windows.Visibility.Hidden);
            // Change the window title
            this.Title = "Fishing game for Windows 1.0";
            // Make the fishes visible
            setFishesVisibility(System.Windows.Visibility.Visible);

        }

        private void showCodeBreaker()
        {
            // Hide all the controls related to the fishes game
            setFishesVisibility(System.Windows.Visibility.Hidden);
            // Change the window title
            this.Title = "CodeBreaker Application";
            // Make the code breaker controls visible
            setCodeBreakerVisibility(System.Windows.Visibility.Visible);

        }

        private bool checkCodeChar(char pcChar, int piCharNumber)
        {
            // Returns a bool value indicating whether the piCharNumber position of the code is the pcChar received.
            return (Code[piCharNumber] == pcChar);
        }

        private void RunWorkerCompletedProcedure(object sender, RunWorkerCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                // Obtain the CodeBreakerResult instance contained in the Result property of e parameter.
                CodeBreakerResult loResult = (CodeBreakerResult)e.Result;
                int i;

                // Iterate through the parts of the result resolved by this BackgroundWorker.
                for (i = loResult.FirstCharNumber; i <=
                     loResult.LastCharNumber; i++)
                {
                    // The process has finishes, therefore the ProgressBar control must show a 100%.
                    prloProgressChar[i].Value = 100;
                    // Show the part of the broken code in the label
                    OutputCharLabels[i].Text = loResult.
                    BrokenCode[i - loResult.FirstCharNumber].ToString();
                }
            }
        }

        private void bakCodebreaker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RunWorkerCompletedProcedure(sender, e);
        }

        private void bakCodebreaker2_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RunWorkerCompletedProcedure(sender, e);
        }

        private void bakCodebreaker3_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RunWorkerCompletedProcedure(sender, e);
        }

        private void bakCodebreaker4_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            RunWorkerCompletedProcedure(sender, e);
        }

        private void ProgressChangedProcedure(object sender, ProgressChangedEventArgs e)
        {
            // This variable will hold a CodeBreakerProgress instance
            CodeBreakerProgress loCodeBreakerProgress =(CodeBreakerProgress)e.UserState;

            // Update the corresponding ProgressBar with the percentage received as a parameter.
            prloProgressChar[loCodeBreakerProgress.CharNumber].Value = loCodeBreakerProgress.PercentageCompleted;

            // Update the corresponding Label with the character being processed.
            OutputCharLabels[loCodeBreakerProgress.CharNumber].Text = ((char)loCodeBreakerProgress.CharCode).ToString();
        }

        private void bakCodebreaker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressChangedProcedure(sender, e);
        
            /*
            // This variable will hold a CodeBreakerProgress instance
            CodeBreakerProgress loCodeBreakerProgress =
             (CodeBreakerProgress)e.UserState; 

            // Update the corresponding ProgressBar with the percentage received in the as a parameter.
            prloProgressChar[loCodeBreakerProgress.CharNumber].Value =
             loCodeBreakerProgress.PercentageCompleted;
            // Update the corresponding Label with the character being processed.
            OutputCharLabels[loCodeBreakerProgress.CharNumber].Text =
             ((char)loCodeBreakerProgress.CharCode).ToString();
            */
        }

        private void bakCodebreaker2_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressChangedProcedure(sender, e);
        }

        private void bakCodebreaker3_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressChangedProcedure(sender, e);
        }

        private void bakCodebreaker4_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            ProgressChangedProcedure(sender, e);
        }

        private void bakCodebreaker_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWorkProcedure(sender, e);
        
            /*
            // This code will break the simulated code.
            // This variable will hold a number to iterate from 1 to 65,535 - Unicode character set.
            int i;
            // This variable will hold a number to iterate from 0 to 3 (the characters positions in the code to be broken).
            int liCharNumber;
            // This variable will hold a char generated from the number in i
            char lcChar;
            // This variable will hold the current Label control that shows the char position being decoded.
            TextBlock loOutputCharCurrentLabel;

            for (liCharNumber = 0; liCharNumber < 4; liCharNumber++)
            {
                // This variable will hold a CodeBreakerProgress instance
                CodeBreakerProgress loCodeBreakerProgress =
                    new CodeBreakerProgress();
                // This variable will hold the last percentage of the iteration completed.
                int liOldPercentageCompleted;

                liOldPercentageCompleted = 0;

                loOutputCharCurrentLabel =
                OutputCharLabels[liCharNumber];
                // This loop will run 65,536 times
                for (i = 0; i <= 65535; i++)
                {
                    if (bakCodebreaker.CancellationPending)
                    {
                        // The user requested to cancel the process
                        e.Cancel = true;
                        return;
                    }

                    // myChar holds a Unicode char
                    lcChar = (char)(i);
                    //loOutputCharCurrentLabel.Text = lcChar.ToString();

                    // The percentage completed is calculated and stored in the PercentageCompleted property.
                    loCodeBreakerProgress.PercentageCompleted =
                     (int)((i * 100) / 65535);
                    loCodeBreakerProgress.CharNumber = liCharNumber;
                    loCodeBreakerProgress.CharCode = i;

                    if (loCodeBreakerProgress.PercentageCompleted >
                        liOldPercentageCompleted)
                    {
                        // The progress is reported only when it changes with regard to the last one (liOldPercentageCompleted).
                        bakCodebreaker.ReportProgress(loCodeBreakerProgress
                         .PercentageCompleted, loCodeBreakerProgress);
                        // The old percentage completed is now the percentage reported.
                        liOldPercentageCompleted = loCodeBreakerProgress.PercentageCompleted;
                    }

                    System.Threading.Thread.Sleep(1);
                    if (checkCodeChar(lcChar, liCharNumber))
                    {
                        // The code position was found
                        loCodeBreakerProgress.PercentageCompleted = 100;
                        bakCodebreaker.ReportProgress(loCodeBreakerProgress.
                            PercentageCompleted, loCodeBreakerProgress);

                        
                        break;

                    }
                }

               // MessageBox.Show("The code has been decoded successfully.",
               //                  this.Title);
            }
             */
          
        }

        private void bakCodebreaker2_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWorkProcedure(sender, e);
        }

        private void bakCodebreaker3_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWorkProcedure(sender, e);
        }

        private void bakCodebreaker4_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWorkProcedure(sender, e);
        }

        private void DoWorkProcedure(object sender, DoWorkEventArgs e)
        {
            
            CodeBreakerParameters loCodeBreakerParameters = (CodeBreakerParameters)e.Argument;


            // This code will break the simulated code.
            // This variable will hold a number to iterate from 1 to 65,535 - Unicode character set.
            int i;
            // This variable will hold a number to iterate from 0 to 3 (the characters positions in the code to be broken).
            int liCharNumber;
            // This variable will hold a char generated from the number in i
            char lcChar;
            // This variable will hold the current Label control that shows the char position being decoded.
            TextBlock loOutputCharCurrentLabel;

            for (liCharNumber = loCodeBreakerParameters.FirstCharNumber;liCharNumber <= loCodeBreakerParameters.LastCharNumber;liCharNumber++)
            {
                // This variable holds the last Unicode character to be processed
                int liTotal = loCodeBreakerParameters.MaxUnicodeCharCode;

                // This variable will hold the broken code
                string lsBrokenCode = "";

                // This variable will hold a CodeBreakerProgress instance
                CodeBreakerProgress loCodeBreakerProgress = new CodeBreakerProgress();
                // This variable will hold the last percentage of the iteration completed.
                int liOldPercentageCompleted;

                liOldPercentageCompleted = 0;

                loOutputCharCurrentLabel =
                OutputCharLabels[liCharNumber];
                // This loop will run 65,536 times
                for (i = 0; i <= 65535; i++)
                {
                    if (((BackgroundWorker)sender).CancellationPending)
                    {
                        // The user requested to cancel the process
                        e.Cancel = true;
                        return;
                    }

                    // myChar holds a Unicode char
                    lcChar = (char)(i);
                    //loOutputCharCurrentLabel.Text = lcChar.ToString();

                    // The percentage completed is calculated and stored in the PercentageCompleted property.
                    loCodeBreakerProgress.PercentageCompleted =
                     (int)((i * 100) / 65535);
                    loCodeBreakerProgress.CharNumber = liCharNumber;
                    loCodeBreakerProgress.CharCode = i;

                    if (loCodeBreakerProgress.PercentageCompleted >
                        liOldPercentageCompleted)
                    {
                        // The progress is reported only when it changes with regard to the last one (liOldPercentageCompleted).
                        ((BackgroundWorker)sender).ReportProgress(loCodeBreakerProgress.PercentageCompleted, loCodeBreakerProgress);

                        // The old percentage completed is now the percentage reported.
                        liOldPercentageCompleted = loCodeBreakerProgress.PercentageCompleted;
                    }

                    System.Threading.Thread.Sleep(1);
                    if (checkCodeChar(lcChar, liCharNumber))
                    {
                        // The code position was found

                        // The broken code is concatenated in lsBrokenCode
                        lsBrokenCode += lcChar.ToString();

                        loCodeBreakerProgress.PercentageCompleted = 100;

                        ((BackgroundWorker)sender).ReportProgress(loCodeBreakerProgress.PercentageCompleted, loCodeBreakerProgress);

                        break;

                    }
                }

                // MessageBox.Show("The code has been decoded successfully.",
                //                  this.Title);

                // Create a new instance of the CodeBreakerResult class and set its properties' values
                CodeBreakerResult loResult = new CodeBreakerResult();
                loResult.FirstCharNumber = loCodeBreakerParameters.FirstCharNumber;
                loResult.LastCharNumber = loCodeBreakerParameters.LastCharNumber;
                loResult.BrokenCode = lsBrokenCode;

                // Return a CodeBreakerResult instance in the Result property
                e.Result = loResult;    

            }
        }

        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            // Disable the Start button
            btnStart.IsEnabled = false;
            // Enable the Stop button
            btnStop.IsEnabled = true;

            // Start running the code programmed in each BackgroundWorker DoWork event handler in a new independent thread and return control to the application's main thread
            // First, create the CodeBreakerParameters for each BackgroundWorker and set its parameters.
            CodeBreakerParameters loParameters1 = new CodeBreakerParameters();
            CodeBreakerParameters loParameters2 = new CodeBreakerParameters();
            CodeBreakerParameters loParameters3 = new CodeBreakerParameters();
            CodeBreakerParameters loParameters4 = new CodeBreakerParameters();
            loParameters1.MaxUnicodeCharCode = 32000;
            loParameters1.FirstCharNumber = 0;
            loParameters1.LastCharNumber = 0;
            loParameters2.MaxUnicodeCharCode = 32000;
            loParameters2.FirstCharNumber = 1;
            loParameters2.LastCharNumber = 1;
            loParameters3.MaxUnicodeCharCode = 32000;
            loParameters3.FirstCharNumber = 2;
            loParameters3.LastCharNumber = 2;
            loParameters4.MaxUnicodeCharCode = 32000;
            loParameters4.FirstCharNumber = 3;
            loParameters4.LastCharNumber = 3;
            bakCodebreaker.RunWorkerAsync(loParameters1);
            bakCodebreaker2.RunWorkerAsync(loParameters2);
            bakCodebreaker3.RunWorkerAsync(loParameters3);
            bakCodebreaker4.RunWorkerAsync(loParameters4);


        }

        private void btnHide_Click(object sender, RoutedEventArgs e)
        {
            // Hide the Codebreaker and show the fishes game
            showFishes();
        }

        private void btnGameOver_Click(object sender, RoutedEventArgs e)
        {
            // Hide the fishes game and show the Codebreaker
            showCodeBreaker();
        }

        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            // Disable the Stop button
            btnStop.IsEnabled = false;
            // Enable the Start button
            btnStart.IsEnabled = true;
            
            //Call the CancelAsync method to cancel the process for all 4 background workers.
            bakCodebreaker.CancelAsync();
            bakCodebreaker2.CancelAsync();
            bakCodebreaker3.CancelAsync();
            bakCodebreaker4.CancelAsync();


        }

    }

}
