﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading;
using System.ComponentModel;

namespace SMSEncryption
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Thread proThreadEncryption;             // The thread
        private List<Thread> prloThreadList;            // The threads list
        private List<string> prlsSMSToEncrypt;          // The string list with SMS messages to encrypt (input)
        private List<List<string>> prlsEncryptedSMS;    // The string list with SMS messages encrypted (output)
        private List<int> priLastEncryptedString;       // The number of the last encrypted string 
        private int priLastEncryptedStringShown;        // The number of the last encrypted string shown in the UI
        private int priOldLastEncryptedStringShown;     // The number of the previous last encrypted string shown in the UI
        private int priProcessorCount = Environment.ProcessorCount; // The number of processors or cores available in the computer for this application

        private BackgroundWorker bakShowEncryptedStrings = new BackgroundWorker();

        public MainWindow()
        {
            InitializeComponent();

            //Register the DoWork and ProgressChanged event handler.
            bakShowEncryptedStrings.DoWork += bakShowEncryptedStrings_DoWork;
            bakShowEncryptedStrings.ProgressChanged += bakShowEncryptedStrings_ProgressChanged;

            //Tell the background worker to report progress.
            bakShowEncryptedStrings.WorkerReportsProgress = true;
            bakShowEncryptedStrings.WorkerSupportsCancellation = true;
        }

        private void butTest_Click(object sender, RoutedEventArgs e)
        {
            //Clear Encrypted box before re-populating it.
            txtEncryptedSMS.Text = "";

            // The encrypted text
            string lsEncryptedText;

            // For each line in txtOriginalSMS TextBox
            int lineCount = txtOriginalSMS.LineCount;
            for (int line = 0; line < lineCount; line++)
            {
                lsEncryptedText = EncryptProcedures.Encrypt(txtOriginalSMS.GetLineText(line));
                // Append a line with the Encrypted text
                txtEncryptedSMS.AppendText(lsEncryptedText + Environment.NewLine);
                // Append a line with the Encrypted text decrypted to test everything is as expected
                txtEncryptedSMS.AppendText(EncryptProcedures.Decrypt(lsEncryptedText) + Environment.NewLine);
            }
        }

        private void butRunInThread_Click(object sender, RoutedEventArgs e)
        {
            // Prepare everything the thread needs from the UI
            // For each line in txtOriginalSMS TextBox
            prlsSMSToEncrypt = new List<string>(txtOriginalSMS.LineCount);

            // Add the lines in txtOriginalSMS TextBox
            int lineCount = txtOriginalSMS.LineCount;
            for (int line = 0; line < lineCount; line++)
            {
                prlsSMSToEncrypt.Add(txtOriginalSMS.GetLineText(line));
            }
            
            // Thread number
            int liThreadNumber;

            // Create the thread list and string lists
            prloThreadList = new List<Thread>(priProcessorCount);
            prlsEncryptedSMS = new List<List<string>>(priProcessorCount);
            priLastEncryptedString = new List<int>(priProcessorCount);

            // Initialize the threads
            for (liThreadNumber = 0; liThreadNumber < priProcessorCount;
                 liThreadNumber++)
            {
                // Just to occupy the number
                prlsEncryptedSMS.Add(new List<string>());
                // Just to occupy the number
                priLastEncryptedString.Add(0);
                // Add the new thread, with a parameterized start (to allow parameters).
                prloThreadList.Add(new Thread(new ParameterizedThreadStart
                                   (ThreadEncryptProcedure)));
            }

            // Now, start the threads
            for (liThreadNumber = 0; liThreadNumber < priProcessorCount;
                 liThreadNumber++)
            {
                prloThreadList[liThreadNumber].Start(liThreadNumber);
            }


            bakShowEncryptedStrings.RunWorkerAsync();

            // Start running the thread
            //proThreadEncryption.Start();

        }

        /*
        private void butRunInThread_Click(object sender, RoutedEventArgs e)
        {
            // Prepare everything the thread needs from the UI
            // For each line in txtOriginalSMS TextBox
            prlsSMSToEncrypt = new List<string>(txtOriginalSMS.LineCount);

            // Add the lines in txtOriginalSMS TextBox
            int lineCount = txtOriginalSMS.LineCount;
            for (int line = 0; line < lineCount; line++)
            {
                prlsSMSToEncrypt.Add(txtOriginalSMS.GetLineText(line));
            }
            // Create the new Thread and use the ThreadEncryptProcedure method
            proThreadEncryption = new Thread(new ThreadStart(ThreadEncryptProcedure));

            bakShowEncryptedStrings.RunWorkerAsync();

            // Start running the thread
            proThreadEncryption.Start();

            // Join the independent thread to this thread to wait until ThreadProc ends
            //proThreadEncryption.Join();

            // When the thread finishes running this is the next line that is going to be executed
            // Copy the string List generated by the thread
            //foreach (string lsEncryptedText in prlsEncryptedSMS)
            //{
                //Append a line with the Encrypted text
            //    txtEncryptedSMS.AppendText(lsEncryptedText +
            //     Environment.NewLine);
            //}
        }
        */

        /*
        private void butRunInThread_Click(object sender, RoutedEventArgs e)
        {
            // Give the main thread a name
            Thread.CurrentThread.Name = "Main thread";

            // Prepare everything the thread needes from the UI
            // For each line in txtOriginalSMS TextBox
            prlsSMSToEncrypt = new List<string>(txtOriginalSMS.LineCount);

            // Add the lines in txtOriginalSMS TextBox
            int lineCount = txtOriginalSMS.LineCount;
            for (int line = 0; line < lineCount; line++)
            {
                prlsSMSToEncrypt.Add(txtOriginalSMS.GetLineText(line));
            }

            // Thread number
            int liThreadNumber;
            // Create the thread list and string lists
            prloThreadList = new List<Thread>(priProcessorCount);
            prlsEncryptedSMS = new List<List<string>>(priProcessorCount);
            priLastEncryptedString = new List<int>(priProcessorCount);

            // Initialize the threads
            for (liThreadNumber = 0; liThreadNumber < priProcessorCount; liThreadNumber++)
            {
                // Just to occupy the number
                prlsEncryptedSMS.Add(new List<string>());
                // Just to occupy the number
                priLastEncryptedString.Add(0);
                // Add the new thread, with a parameterized start (to allow parameters)
                prloThreadList.Add(new Thread(new ParameterizedThreadStart(ThreadEncryptProcedure)));
            }

            // Now, start the threads
            for (liThreadNumber = 0; liThreadNumber < priProcessorCount; liThreadNumber++)
            {
                // Give the thread a name
                prloThreadList[liThreadNumber].Name = "Encryption #" + liThreadNumber.ToString();
                // Start the thread
                prloThreadList[liThreadNumber].Start(liThreadNumber);
            }

            // Start the BackgroundWorker with an asynchronous execution
            bakShowEncryptedStrings.RunWorkerAsync();
        }
        */

        /*
        private void bakShowEncryptedStrings_DoWork(object sender, DoWorkEventArgs e)
        {
            // Initialize the last encrypted string shown
            priLastEncryptedStringShown = 0;
            // Initialize the last encrypted string shown before
            priOldLastEncryptedStringShown = 0;
            // The iteration
            int i;
            // The last encrypted string (saved locally to avoid changes in the middle of the iteration)
            int liLast;

            // Wait until proThreadEncryption begins
            while ((priLastEncryptedString < 1))
            {
                // Sleep the thread for 10 milliseconds)
                Thread.Sleep(10);
            }

            while (proThreadEncryption.IsAlive || (priLastEncryptedString
                   > priLastEncryptedStringShown))
            {
                liLast = priLastEncryptedString;
                if (liLast != priLastEncryptedStringShown)
                {
                    ((BackgroundWorker)sender).ReportProgress(liLast);
                    priLastEncryptedStringShown = liLast;
                }

                // Sleep the thread for 1 seconds (1000 milliseconds)
                Thread.Sleep(1000);
             }

        }
        */

        private void bakShowEncryptedStrings_DoWork(object sender, DoWorkEventArgs e)
        {
            // Give the BackgroundWorker thread a name
            Thread.CurrentThread.Name = "bakShowEncryptedStrings";
            // Initialize the last encrypted string shown
            priLastEncryptedStringShown = 0;
            // Initialize the last encrypted string shown before
            priOldLastEncryptedStringShown = 0;
            // The iteration
            int i;
            // The last encrypted string (saved locally to avoid changes in the middle of the iteration)
            int liLast;

            // Wait until proThreadEncryption begins
            while ((priLastEncryptedString[0] < 1))
            {
                // Sleep the thread for 10 milliseconds)
                Thread.Sleep(10);
            }

            while (prloThreadList[0].IsAlive || (priLastEncryptedString[0] > priLastEncryptedStringShown))
            {
                liLast = priLastEncryptedString[0];
                if (liLast != priLastEncryptedStringShown)
                {
                    ((BackgroundWorker)sender).ReportProgress(liLast);
                    priLastEncryptedStringShown = liLast;
                }

                // Sleep the thread for 2 seconds (2000 milliseconds)
                Thread.Sleep(2000);
            }
        }      

        /*
        private void bakShowEncryptedStrings_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // The iteration
            int i;
            // Show the number of SMS messages encrypted by the concurrent proThreadEncryption thread.
            lblNumberOfSMSEncrypted.Content = priLastEncryptedString.ToString();
            // Append each new string, from priOldLastEncryptedStringShown to the received parameter in e.ProgressPercentage - 1.
            for (i = priOldLastEncryptedStringShown;
                 i < (int)e.ProgressPercentage; i++)
            {
                // Append the string to the txtEncryptedSMS TextBox
                txtEncryptedSMS.AppendText(prlsEncryptedSMS[i]
                 + Environment.NewLine);

            }
            // Update the old last encrypted string shown
            priOldLastEncryptedStringShown = priLastEncryptedStringShown;

        }
        */

        private void bakShowEncryptedStrings_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            // The iteration
            int i;
            // Show the number of SMS messages encrypted by the concurrent proThreadEncryption thread
            lblNumberOfSMSEncrypted.Content = priLastEncryptedString[0].ToString();

            // Append each new string, from priOldLastEncryptedStringShown to the received parameter in e.ProgressPercentage - 1
            for (i = priOldLastEncryptedStringShown; i < (int)e.ProgressPercentage; i++)
            {
                // Append the string to the txtEncryptedSMS TextBox
                txtEncryptedSMS.AppendText(prlsEncryptedSMS[0][i] + Environment.NewLine);
            }
            // Update the old last encrypted string shown
            priOldLastEncryptedStringShown = priLastEncryptedStringShown;
        }

        /*
        private void ThreadEncryptProcedure()
        {
            priLastEncryptedString = 0;

            string lsEncryptedText;

            //Initialize the encrypted array to the size of the array to encrypt.
            prlsEncryptedSMS = new List<string>(prlsSMSToEncrypt.Count);

            // steps the thread number
            string lsText;
            // Iterate through each string in the prlsSMSToEncrypt string list stepping by priProcessorCount
            // To distribute the work among each concurrent thread
            for (int i = 0; i < prlsSMSToEncrypt.Count; i++)
            {
                lsText = prlsSMSToEncrypt[i];
                lsEncryptedText = EncryptProcedures.Encrypt(lsText);
                // Append a string with the Encrypted text
                prlsEncryptedSMS.Add(lsEncryptedText);
                priLastEncryptedString++;
            }
        }
         */
        
        private void ThreadEncryptProcedure(object poThreadParameter)
        {
            string lsEncryptedText;
            // Retrieve the thread number received in object poThreadParameter
            int liThreadNumber = (int)poThreadParameter;
            // ThreadNumber + 1
            int liStringNumber;

            // Create a new string list for the prlsSMSToEncrypt corresponding to the thread
            prlsEncryptedSMS[liThreadNumber] = new List<string>((prlsSMSToEncrypt.Count / priProcessorCount));

            priLastEncryptedString[liThreadNumber] = 0;

            liStringNumber = 0;
            int i;
            // steps the thread number
            string lsText;
            // Iterate through each string in the prlsSMSToEncrypt string list stepping by priProcessorCount
            // To distribute the work among each concurrent thread
            for (i = liThreadNumber; i < prlsSMSToEncrypt.Count; i += priProcessorCount)
            {
                lsText = prlsSMSToEncrypt[i];
                lsEncryptedText = EncryptProcedures.Encrypt(lsText);
                // Append a string with the Encrypted text
                prlsEncryptedSMS[liThreadNumber].Add(lsEncryptedText);

                priLastEncryptedString[liThreadNumber]++;
                liStringNumber++;
            }
        }
         
    }
}
