﻿
using System;
using System.Threading;

public class ProducerConsumer
{
    public static void Main(String[] args)
    {
        int result = 0;   // Results output
        Cell cell = new Cell();

        Producer producer = new Producer(cell, 5);  
        Consumer consumer = new Consumer(cell, 5);  

        Thread producerThread = new Thread(new ThreadStart(producer.ThreadRun));
        Thread consumerThread = new Thread(new ThreadStart(consumer.ThreadRun));

        try
        {
            producerThread.Start();
            consumerThread.Start();

            // Join both threads.
            producerThread.Join();   
            consumerThread.Join();

        }
        catch (ThreadStateException e)
        {
            System.Diagnostics.Debug.WriteLine(e);  // Output text of exception.
            result = 1;            // Set result to indicate an error.
        }
        catch (ThreadInterruptedException e)
        {
            System.Diagnostics.Debug.WriteLine(e);  // Output text noting an interruption.
            result = 1;            // Set result to indicate an error.
        }

        Environment.ExitCode = result;
    }
}

public class Producer
{
    Cell cell;        
    int quantity = 1;  

    public Producer(Cell box, int request)
    {
        cell = box;          
        quantity = request;  
    }
    public void ThreadRun()
    {
        for (int looper = 1; looper <= quantity; looper++)
            cell.Write(looper);  // "producing"
    }
}

public class Consumer
{
    Cell cell;        
    int quantity = 1;  

    public Consumer(Cell box, int request)
    {
        cell = box;         
        quantity = request;  
    }
    public void ThreadRun()
    {
        int value;
        for (int looper = 1; looper <= quantity; looper++)
            value = cell.Read();    // Consume the result by putting it in value
    }
}

public class Cell
{
    int cellContents;      
    bool State = false;  
    public int Read()
    {
        lock (this)   // Synchronizing block of code.
        {
            if (!State)
            {            // Wait until Cell.Write is done producing
                try
                {
                    Monitor.Wait(this);     // Waits for the Monitor.Pulse in Write
                }
                catch (SynchronizationLockException e)
                {
                    System.Diagnostics.Debug.WriteLine(e);
                }
                catch (ThreadInterruptedException e)
                {
                    System.Diagnostics.Debug.WriteLine(e);
                }
            }
            System.Diagnostics.Debug.WriteLine(String.Format("Consumed cell item {0}", cellContents));

            State = false;    // Consumtion is done.

            Monitor.Pulse(this);   // Pulse tells Cell.Write thatCell.Read is finished.
        }   

        return cellContents;
    }

    public void Write(int n)
    {
        lock (this)  // Synchronization block
        {
            if (State)
            {      // Wait until Cell.Read is done consumption.
                try
                {
                    Monitor.Wait(this);   // Wait for the Monitor.Pulse in Read.
                }
                catch (SynchronizationLockException e)
                {
                    System.Diagnostics.Debug.WriteLine(e);
                }
                catch (ThreadInterruptedException e)
                {
                    System.Diagnostics.Debug.WriteLine(e);
                }
            }
            cellContents = n;
            System.Diagnostics.Debug.WriteLine(String.Format("Produced cell item {0}", cellContents));
            State = true;    // Set State to indicate production is done.

            Monitor.Pulse(this);  // Pulse tells Cell.Read that Cell.Write is finished.

        }   
    }
}
